# Copyright (C) 2009 Raul Mircea Gigea <raul.gigea@gmail.com>

#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

#!/usr/local/bin/python
from twisted.words.protocols.jabber import client as jabberclient, jid
from twisted.words.xish import domish
from twisted.internet import reactor

import sys, fcntl, os, threading, urllib
from threading import Thread

from deluge.ui.console.main import BaseCommand, match_torrents
from deluge.ui.client import aclient as client
from deluge.ui.client import sclient
import deluge.common as common
import deluge.error as error
from deluge.ui.console import mapping

myJid = None
myPass = None
thexmlstream = None
deluge = None
  
helpStringHTML = "<html xmlns='http://jabber.org/protocol/xhtml-im'><body xmlns='http://www.w3.org/1999/xhtml'>Commands:<br/><ul>" +  \
"<li><span style='font-weight: bold;'>status</span> - Prints current status information about all downloading torrents</li><br/>" + \
"<li><span style='font-weight: bold;'>add [URL of Torrent]</span> - Gets Torrent File from URL and starts downloading</li><br/>" + \
"<li><span style='font-weight: bold;'>del [Id of Torrent]</span> - Deletes Torrent File with given Id</li><br/>" + \
"<li><span style='font-weight: bold;'>help</span> - This command</li><br/>" + \
"</ul></body></html>"

helpString = "status - Prints current status information about all downloading torrents\n" + \
"add [URL of Torrent] - Gets Torrent File from URL and starts downloading\n" + \
"del [Id of Torrent] - Deletes Torrent File with given Id\n" + \
"help - This command\n"

dalock = threading.Lock()
outgoingMessages = []
myBuddies = []

class delugeConn:
  def __init__(self):
    sclient.set_core_uri()
    self.status_keys = ["state", "save_path", "tracker", "next_announce",
        "name", "total_size", "progress", "num_seeds", "total_seeds",
        "num_peers", "total_peers", "eta", "download_payload_rate", "upload_payload_rate",
        "ratio", "distributed_copies", "num_pieces", "piece_length", "total_done",
        "files", "num_files", "file_priorities", "file_progress", "peers", "is_seed", "hash" ]

  def get_all_torrent_infos_short(self):
    global status_keys
    output = ""

    try:
      torrents = match_torrents()
      for torrent in torrents:
        #print('getting torrent status for %s'%torrent)
        state = sclient.get_torrent_status(torrent, self.status_keys)
        output += '%s - [%s]'%(state['name'], (common.fsize(state['total_done']) + "/" + common.fsize(state['total_size'])) )
        if state['state'] == 'Downloading':
          output += ' [%s]'%common.fspeed(state['download_payload_rate'])
          output += ' [%s]'%common.ftime(state['eta'])
        if state['state'] in ('Seeding', 'Paused'):
          output += ' [Done]'
        output += "\n"
    except error.NoCoreError:
      print('disconnected from deluge. reconecting...')
      sclient.set_core_uri()
      output += "Disconnected From Deluge"

    return output


  def get_all_torrent_infos(self):
    global status_keys
    output = ""

    def _got_torrent_status(state):
      print('state=%s'%`state`)

    try:
      torrents = match_torrents()
      for torrent in torrents:
        #print('getting torrent status for %s'%torrent)
        state = sclient.get_torrent_status(torrent, self.status_keys)
        output += 'Name: %s\n'%state['name']
        output += 'Id: %s\n'%state['hash']
        output += 'Path: %s\n'%state['save_path']
        output += 'Progress: %s\n'% (common.fsize(state['total_done']) + "/" + common.fsize
  (state['total_size']))
        output += 'Status: %s\n'%state['state']
        if state['state'] == 'Downloading':
          output += 'Download Speed: %s\n'%common.fspeed(state['download_payload_rate'])
        if state['state'] in ('Downloading', 'Seeding'):
          output += 'Upload Speed: %s\n'%common.fspeed(state['upload_payload_rate'])
        output += 'ETA: %s\n'%common.ftime(state['eta'])
        output += "\n"
    except error.NoCoreError:
      print('disconnected from deluge. reconecting...')
      sclient.set_core_uri()
      output += "Disconnected From Deluge"
       

    return output
  
  def get_all_torrent_infos_html(self):
    global status_keys
    output = "<html xmlns='http://jabber.org/protocol/xhtml-im'><body xmlns='http://www.w3.org/1999/xhtml'>"
    
    try:
      torrents = match_torrents()
      for torrent in torrents:
        #print('getting torrent status for %s'%torrent)
        state = sclient.get_torrent_status(torrent, self.status_keys)
        output += "<ul>"
        output += "<li><span style='font-weight: bold; color: #0000aa;'>%s:</span> %s</li><br/>"%('Name', state['name'])
        output += "<li><span style='font-weight: bold; color: #0000aa;'>%s:</span> %s</li><br/>"%('Id', state['hash'])
        output += "<li><span style='font-weight: bold; color: #0000aa;'>%s:</span> %s <a href='afp://192.168.1.100/%s/%s'>Show In Finder</a> </li><br/>"%('Path', state['save_path'], urllib.quote("/".join(state['save_path'].split("/")[2:])) ,urllib.quote(state['files'][0]['path'].split('/')[0]))
        output += "<li><span style='font-weight: bold; color: #00aa00;'>%s:</span> %s</li><br/>"%('Progress', (common.fsize(state['total_done']) + "/" + common.fsize(state['total_size'])) )
        output += "<li><span style='font-weight: bold; color: #00aa00;'>%s:</span> %s</li><br/>"%('Status', state['state'])
        if state['state'] == 'Downloading':
          output += "<li><span style='font-weight: bold; color: #00aa00;'>%s:</span> %s</li><br/>"%('Download Speed', common.fspeed(state['download_payload_rate']))
        if state['state'] in ('Downloading', 'Seeding'):
          output += "<li><span style='font-weight: bold; color: #00aa00;'>%s:</span> %s</li><br/>"%('Upload Speed', common.fspeed(state['upload_payload_rate']))
        output += "<li><span style='font-weight: bold; color: #00aa00;'>%s:</span> %s</li><br/>"%('ETA', common.ftime(state['eta']))
        output += "<br/></ul>"
    except error.NoCoreError:
      print('disconnected from deluge. reconecting...')
      sclient.set_core_uri()
      output += "Disconnected From Deluge"

    output += "</body></html>"

    return output
  
#message.addRawXml("<html xmlns='http://jabber.org/protocol/xhtml-im'><body xmlns='http://www.w3.org/1999/xhtml'><span style='font-weight: bold;'>foo</span> bar</body></html>")

  def add_url(self, url):
    try:
      sclient.add_torrent_url(url, None)
    except error.NoCoreError:
      print('disconnected from deluge. reconecting...')
      sclient.set_core_uri()
      sclient.add_torrent_url(url, None)

  def remove_torrent(self, torrent):
    try:
      sclient.remove_torrent([torrent], False)
    except error.NoCoreError:
      print('disconnected from deluge. reconecting...')
      sclient.set_core_uri()
      sclient.remove_torrent([torrent], False)

  def get_torrent_name(self,torrent):
    try:
      return mapping.get_names([torrent])[0][1]
    except IndexError:
      return 'No Torrent Found for Id:%s'%torrent
    except error.NoCoreError:
      sclient.set_core_uri()
      return 'Disconnected From Deluge.'
    


count = 0

def updateStatus():
    global thexmlstream, deluge,count
    count += 1
    presence = domish.Element(('jabber:client','presence'))
    presence.addElement("show", "", "available")
    presence.addElement("status", "", deluge.get_all_torrent_infos_short())
    thexmlstream.send(presence)
    #reactor.callLater(10, updateStatus)


def authd(xmlstream):
    global thexmlstream, deluge,myJid
    print "[JABBER] Authenticated"
    thexmlstream = xmlstream
    updateStatus()

    xmlstream.addObserver('/message',  gotMessage)
    xmlstream.addObserver('/presence', debug)
    xmlstream.addObserver('/iq',       gotIq)

    # Get Roster

    iq = domish.Element(('', 'iq'))
    iq['from'] = myJid.full()
    iq['type'] = 'get'
    #iq['id']='roster_1'
    iq.addElement("query", "jabber:iq:roster", "")

    xmlstream.send(iq)

def gotIq(el):
  global myBuddies,myJid
  if el['type'] == 'result' and el['to']==myJid.full() and el['from']==myJid.userhost():
    print('got Iq result')
    query = list(el.elements())[0]
    for item in query.elements():
      print('authorizing %s'%item['jid'])
      myBuddies += [item['jid']]

def checkWorkQueue():
    global dalock, outgoingMessages,myJid
    dalock.acquire()

    for message in outgoingMessages:
      dest = message[:message.find("<msg>")]
      msg = message[message.find("<msg>")+5:]
      send(myJid, dest, msg, None)
      print('Sending to %s message %s'%(dest,msg))

    outgoingMessages = []

    dalock.release()
    updateStatus()

    reactor.callLater(10, checkWorkQueue)

def send(author, to, msg, msgHTML):
  global thexmlstream
  message = domish.Element(('jabber:client','message'))
  message["to"] = jid.JID(to).full()
  message["from"] = author.full()
  message["type"] = "chat"
  message.addElement("body", "jabber:client", msg )
  message.addRawXml(msgHTML)
  #message.addRawXml("<html xmlns='http://jabber.org/protocol/xhtml-im'><body xmlns='http://www.w3.org/1999/xhtml'><span style='font-weight: bold;'>foo</span> bar</body></html>")
  #print('Sending rawxml: %s'%msgHTML)
  thexmlstream.send(message)

def gotMessage(el):
  global myJid, deluge,helpString, helpStringHTML, myBuddies
  print el.toXml().encode('utf-8')
  #print 'Got message: %s' % str(el.attributes)
  from_id = el["from"]


  
  body = "empty"
  for e in el.elements():
    if e.name == "body":
      body = unicode(e.__str__())
      break
    if e.name == "error":
      print('got some error')
      return
  
  body = body.split(" ")
  
  auth = False

  for buddy in myBuddies:
    if ( from_id.find(buddy) != -1 ):
      auth = True
   
  if auth == False:
    send(myJid, from_id, 'Unauthorized JID:%s'%from_id, None)
    print("DENIED: '%s'"%from_id )
    return

  if 'status' in body:
    #send(myJid, from_id, deluge.get_all_torrent_infos(), deluge.get_all_torrent_infos_html())
    send(myJid, from_id, None, deluge.get_all_torrent_infos_html())
    #send(myJid, from_id, deluge.get_all_torrent_infos(), None)
    return

  if 'add' in body:
    url = body[-1]
    deluge.add_url( url )
    send(myJid, from_id, 'Adding %s'%url, None)
    return

  if 'del' in body:
    url = body[-1]
    send(myJid, from_id, 'Deleting %s'% deluge.get_torrent_name(url), None)
    deluge.remove_torrent( url )
    return
    
  if 'help' in body:
    send(myJid, from_id, helpString, helpStringHTML)
    return

  body = " ".join(body)
  
  if not body == 'empty':
    send(myJid, from_id, "Unknown command: '%s'\nType 'help' for a list of commands."%body, None)

def debug(elem):
    print "="*20
    print elem.toXml().encode('utf-8')
    print "="*20

started = False
connector = None
inited = False

class JabberThread(Thread):
    def __init__(self):
        Thread.__init__(self)
    def connect(account,passw):
        global myJid,myPass
        myJid = jid.JID(account+ '/TwistedWords')
        myPass = passw
        print('connect...')
        jabT = JabberThread()
        jabT.start()
    def disconnect():
        global connector,started
        connector.disconnect()
        reactor.stop()
        started = False
    def sendMessage(to,msg):
        global dalock
        dalock.acquire()
        outgoingMessages = outgoingMessages + "%s<msg>%s</dld>"%(to,msg)
        dalock.release()

    connect = staticmethod(connect)
    disconnect = staticmethod(disconnect)
    sendMessage = staticmethod(sendMessage)

    def run(self):
        print 'Starting Jabber Thread'
        global started, connector, inited,deluge,myJid,myPass
        if not started:
          started = True
          if not inited:
            inited = True
            print 'Initializing Reactor for Jabber'
            deluge = delugeConn()
            factory = jabberclient.basicClientFactory(myJid, myPass)
            factory.addBootstrap('//event/stream/authd',authd)
            connector = reactor.connectTCP('jabber.org',5222,factory)
            reactor.callLater(20, checkWorkQueue)
          # TODO: load jid and so on
          #log.debug('RemoteNotify: Starting Jabber Client.')
          print 'Starting Reactor for Jabber'
          try:
            reactor.run(installSignalHandlers=0)
            print '++++++++++++++++++++++++++RemoteNotify: Jabber Exited Normally.'
          except exceptions.ValueError:
            print '--------------------------RemoteNotify: Jaber exited with some ValueError Exception.'
        else:
          print 'Already Started'

