# Copyright (C) 2009 Raul Mircea Gigea <raul.gigea@gmail.com>

#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
#     The Free Software Foundation, Inc.,
#     51 Franklin Street, Fifth Floor
#     Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#
import os
from deluge.log import LOG as log
from deluge.ui.client import sclient, aclient
from deluge import component
import ui

import deluge.ui.webui.lib.newforms_plus as forms

config_page_manager = component.get("ConfigPageManager")


class RemoteNotifyCfgForm(forms.Form):
    """
    a config form based on django forms.
    see webui.lib.newforms_plus, config_forms, etc.
    """
    #meta:
    title = _("RemoteNotify")

    #load/save:
    def initial_data(self):
        data = sclient.remotenotify_get_config()
        return data

    def save(self, data):
        cfg = dict(data)
        sclient.remotenotify_set_config(cfg)

    #input fields :
    im_account = forms.CharField(label=_("Notifier IM Account"))
    im_pass = forms.CharField(label=_("Notifier IM Password"))
    mail_account = forms.CharField(label=_("Notifier Email Account"))
    mail_smtp = forms.CharField(label=_("Notifier Email SMTP Server"))
    mail_pass = forms.CharField(label=_("Notifier Email Account Password"))

    im_notifyperson1 = forms.CharField(label=_("Notified Person 1 IM Id"))
    im_notifyperson2 = forms.CharField(label=_("Notified Person 2 IM Id"), required=False)
    mail_notifyperson1 = forms.CharField(label=_("Notified Person 1 Email"))
    mail_notifyperson2 = forms.CharField(label=_("Notified Person 2 Email"), required=False)


    def post_html(self):
        "show remotenotify status"

        return ""


class WebUI(ui.UI):
    def __init__(self, plugin_api, plugin_name):
        log.debug("Calling UI init")
        # Call UI constructor
        ui.UI.__init__(self, plugin_api, plugin_name)
        log.debug("RemoteNotify WebUI plugin initalized..")

    def enable(self):
        config_page_manager.register('plugins','remotenotify',RemoteNotifyCfgForm)

    def disable(self):
        config_page_manager.deregister('remotenotify')



